@@ -23,7 +23,6 @@
 
 #include <glib/gi18n.h>
 
-#include "gtksourcecompletion-private.h"
 #include "gtksourceindenter.h"
 #include "gtksourceview.h"
 
@@ -135,25 +134,6 @@ gtk_source_vim_insert_handle_keypress (GtkSourceVimState *state,
 				g_signal_emit_by_name (view, "paste-clipboard");
 				return TRUE;
 
-			case GDK_KEY_n:
-			case GDK_KEY_p:
-			{
-				GtkSourceCompletion *completion = gtk_source_view_get_completion (view);
-
-				if (_gtk_source_completion_get_visible (completion))
-				{
-					_gtk_source_completion_move_cursor (completion,
-									    GTK_MOVEMENT_DISPLAY_LINES,
-									    keyval == GDK_KEY_n ? 1 : -1);
-				}
-				else
-				{
-					gtk_source_completion_show (completion);
-				}
-
-				return TRUE;
-			}
-
 			default:
 				break;
 		}
@@ -503,7 +483,6 @@ gtk_source_vim_insert_dispose (GObject *object)
 	G_OBJECT_CLASS (gtk_source_vim_insert_parent_class)->dispose (object);
 
 	g_clear_pointer (&self->prefix, g_free);
-	g_clear_pointer (&self->suffix, g_free);
 	gtk_source_vim_state_release (&self->history);
 	gtk_source_vim_state_release (&self->motion);
 	gtk_source_vim_state_release (&self->selection_motion);
@@ -566,8 +545,12 @@ gtk_source_vim_insert_set_prefix (GtkSourceVimInsert *self,
 {
 	g_return_if_fail (GTK_SOURCE_IS_VIM_INSERT (self));
 
-	if (g_set_str (&self->prefix, prefix))
+	if (g_strcmp0 (self->prefix, prefix) != 0)
+	{
+		g_free (self->prefix);
+		self->prefix = g_strdup (prefix);
 		g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_PREFIX]);
+	}
 }
 
 void
@@ -576,8 +559,12 @@ gtk_source_vim_insert_set_suffix (GtkSourceVimInsert *self,
 {
 	g_return_if_fail (GTK_SOURCE_IS_VIM_INSERT (self));
 
-	if (g_set_str (&self->suffix, suffix))
+	if (g_strcmp0 (self->suffix, suffix) != 0)
+	{
+		g_free (self->suffix);
+		self->suffix = g_strdup (suffix);
 		g_object_notify_by_pspec (G_OBJECT (self), properties [PROP_SUFFIX]);
+	}
 }
 
 void
