@@ -312,8 +312,7 @@ gtk_source_vim_normal_begin_insert (GtkSourceVimNormal   *self,
 		gtk_source_vim_motion_set_apply_on_leave (GTK_SOURCE_VIM_MOTION (motion), FALSE);
 		gtk_source_vim_insert_set_at (GTK_SOURCE_VIM_INSERT (ret), at);
 		gtk_source_vim_insert_set_motion (GTK_SOURCE_VIM_INSERT (ret),
-		                                  GTK_SOURCE_VIM_MOTION (motion));
-		g_object_unref (motion);
+			                          GTK_SOURCE_VIM_MOTION (motion));
 	}
 
 	gtk_source_vim_state_set_count (ret, count);
@@ -953,7 +952,6 @@ key_handler_search (GtkSourceVimNormal *self,
 	switch (keyval)
 	{
 		case GDK_KEY_slash:
-		case GDK_KEY_KP_Divide:
 			text = "/";
 			break;
 
@@ -1224,7 +1222,6 @@ key_handler_initial (GtkSourceVimNormal *self,
 			case GDK_KEY_j:
 			case GDK_KEY_k:
 			case GDK_KEY_KP_Enter:
-			case GDK_KEY_KP_Multiply:
 			case GDK_KEY_l:
 			case GDK_KEY_L:
 			case GDK_KEY_Left:
@@ -1323,7 +1320,6 @@ key_handler_initial (GtkSourceVimNormal *self,
 				return gtk_source_vim_normal_replace_one (self);
 
 			case GDK_KEY_slash:
-			case GDK_KEY_KP_Divide:
 			case GDK_KEY_question:
 				self->handler = key_handler_search;
 				break;
